@@ -12,6 +12,7 @@ from typing import (
     TypeVar,
     MutableSequence,
     Iterable,
+    Sequence,
 )
 
 
@@ -55,7 +56,7 @@ STATION_DTYPES = OrderedDict(
     channel="nslc_code",
     seed_id=str,
     latitude=float,
-    longitude=float,
+    longitude="longitude",
     elevation=float,
     depth=float,
     azimuth=float,
@@ -73,7 +74,7 @@ DF_TO_INV_DTYPES = OrderedDict(
     location="nslc_code",
     channel="nslc_code",
     latitude=float,
-    longitude=float,
+    longitude="longitude",
     elevation=float,
     depth=float,
     sample_rate=float,
@@ -87,7 +88,7 @@ DF_TO_INV_COLUMNS = tuple(DF_TO_INV_DTYPES)
 EVENT_DTYPES = OrderedDict(
     time="datetime64[ns]",
     latitude=float,
-    longitude=float,
+    longitude="longitude",
     depth=float,
     magnitude=float,
     event_description=str,
@@ -167,9 +168,7 @@ DISTANCE_COLUMN_DTYPES = OrderedDict(
 )
 
 # Columns for dataframe inputs
-DISTANCE_COLUMN_INPUT_DTYPES = OrderedDict(
-    latitude=float, longitude=float, elevation=float
-)
+LOCATION_DTYPE = OrderedDict(latitude=float, longitude=float, elevation=float)
 
 ALT_DISTANCE_COLUMN_DTYPES = OrderedDict(latitude=float, longitude=float, depth=float)
 
@@ -309,6 +308,16 @@ WAVEFORM_DTYPES = OrderedDict(
     sampling_period="timedelta64[ns]",
 )
 
+# dtypes for requesting waveforms
+WAVEFORM_REQUEST_DTYPES = {
+    "network": str,
+    "station": str,
+    "location": str,
+    "channel": str,
+    "starttime": "ops_datetime",
+    "endtime": "ops_datetime",
+}
+
 # The datatypes needed for putting waveform info into HDF5
 WAVEFORM_DTYPES_INPUT = MapProxy(
     {i: _DATETIME_TYPE_MAP.get(v, v) for i, v in WAVEFORM_DTYPES.items()}
@@ -369,7 +378,8 @@ UNSUPPORTED_PARAMS = {"magnitude_type", "events", "contributor"}
 path_types = Union[str, Path]
 
 # number types
-number_type = Union[float, int, np.float, np.int, np.complex]
+# number_type = Union[float, int, np.float, np.int, np.complex]  # deprecated
+number_type = Union[float, int]
 
 # The waveforms processor type
 stream_proc_type = Callable[[Stream], Stream]
@@ -381,7 +391,9 @@ wave_type = Union[Stream, Trace]
 utc_able_type = Union[str, UTCDateTime, float, np.datetime64, pd.Timestamp]
 
 # waveform request type (args for get_waveforms)
-waveform_request_type = Tuple[str, str, str, str, utc_able_type, utc_able_type]
+waveform_request_type = Sequence[
+    Tuple[str, str, str, str, utc_able_type, utc_able_type]
+]
 
 # the signature of obspy fdsn client
 wfcli_type = Callable[[str, str, str, str, UTCDateTime, UTCDateTime], Stream]
@@ -430,7 +442,7 @@ column_function_map_type = Mapping[str, series_func_type]
 bank_subpaths_type = Union[path_types, Iterable[path_types]]
 
 # types for bulk waveform requests
-bulk_waveform_arg_type = List[Tuple[str, str, str, str, UTCDateTime, UTCDateTime]]
+bulk_waveform_arg_type = Union[waveform_request_type, pd.DataFrame]
 
 # types which can be used to slice a numpy array
 slice_types = Union[int, slice, List[int], Tuple[int, ...]]
@@ -526,7 +538,7 @@ GET_STATIONS_UTC_KWARGS = (
 
 # Numpy int types
 NUMPY_INT_TYPES = {
-    np.int,
+    # np.int,  # This is deprecated in numpy 1.20
     np.int32,
     np.int64,
     np.uint,
