@@ -8,7 +8,6 @@ from functools import lru_cache, reduce
 from typing import Any, Optional, Sequence, Mapping, Collection, Iterable, Union
 
 import numpy as np
-import obspy
 import pandas as pd
 from pandas.api.types import is_string_dtype
 
@@ -24,6 +23,7 @@ from obsplus.constants import (
 )
 from obsplus.exceptions import DataFrameContentError
 from obsplus.utils.time import to_datetime64, to_timedelta64, to_utc
+from obsplus.utils.geodetics import map_longitudes
 
 
 def _int_column_to_str(ser, width=2, fillchar="0"):
@@ -42,14 +42,16 @@ OPS_DTYPE_FUNCS = {
     "ops_timedelta": to_timedelta64,
     "utcdatetime": to_utc,
     "nslc_code": _int_column_to_str,
+    "longitude": map_longitudes,
 }
 
 # the dtype of the columns
 OPS_DTYPES = {
-    "ops_datetime": "datetime64",
-    "ops_timedelta": "timedelta64",
-    "utcdatetime": obspy.UTCDateTime,
+    "ops_datetime": "datetime64[ns]",
+    "ops_timedelta": "timedelta64[ns]",
+    "utcdatetime": object,
     "nslc_code": str,
+    "longitude": float,
 }
 
 
@@ -147,8 +149,10 @@ def cast_dtypes(
         'ops_datetime' - call :func:`obsplus.utils.time.to_datetime64` on column
         'ops_timedelta` - call :func:`obsplus.utils.time.to_timedelta64` on column
 
-    Note: this is different from pd.astype because it skips columns which
-    don't exist.
+    Notes
+    -----
+    This function is different from pd.astype because it skips columns which
+    don't exist and handles custom obsplus dtypes.
 
     Parameters
     ----------
@@ -159,19 +163,17 @@ def cast_dtypes(
     inplace
         If true perform operation in place.
     """
-    # get overlapping columns and dtypes
+    df = df if not inplace else df.copy()
+    # get overlapping columns, column functions, and pandas support dtypes
     overlap = set(dtype) & set(df.columns)
-    dtype_codes = {i: dtype[i] for i in overlap}
-    # if the dataframe is empty and has columns use simple astype
-    if df.empty and len(df.columns):
-        dtypes = {i: OPS_DTYPES.get(v, v) for i, v in dtype_codes.items()}
-        return df.astype(dtypes)
-    # else create functions and apply to each column
-    funcs = {
-        i: OPS_DTYPE_FUNCS.get(v, lambda x, y=v: x.astype(y))
-        for i, v in dtype_codes.items()
+    column_funcs = {
+        i: OPS_DTYPE_FUNCS[dtype[i]] for i in overlap if dtype[i] in OPS_DTYPE_FUNCS
     }
-    return apply_funcs_to_columns(df, funcs=funcs, inplace=inplace)
+    supported_dtypes = {i: OPS_DTYPES.get(dtype[i], dtype[i]) for i in overlap}
+    # apply functions defined with custom dtypes
+    if column_funcs:
+        df = apply_funcs_to_columns(df, column_funcs, inplace=inplace)
+    return df.astype(supported_dtypes, copy=False)
 
 
 def order_columns(
@@ -408,7 +410,7 @@ def filter_df(df: pd.DataFrame, **kwargs) -> np.array:
 
 
 def _filter_starttime_endtime(df, starttime=None, endtime=None):
-    """ Filter dataframe on starttime and endtime. """
+    """Filter dataframe on starttime and endtime."""
     bool_index = np.ones(len(df), dtype=bool)
     t1 = to_datetime64(starttime) if starttime is not None else SMALLDT64
     t2 = to_datetime64(endtime) if endtime is not None else LARGEDT64
@@ -421,12 +423,12 @@ def _filter_starttime_endtime(df, starttime=None, endtime=None):
 
 @lru_cache(maxsize=2500)
 def get_regex(seed_str):
-    """ Compile, and cache regex for str queries. """
+    """Compile, and cache regex for str queries."""
     return fnmatch.translate(seed_str)  # translate to re
 
 
 def _column_contains(ser: pd.Series, str_sequence: Iterable[str]) -> pd.Series:
-    """ Test if a str series contains any values in a sequence """
+    """Test if a str series contains any values in a sequence"""
     safe_matches = {re.escape(x) for x in str_sequence}
     return ser.str.contains("|".join(safe_matches)).values
 
@@ -451,7 +453,7 @@ def get_waveforms_bulk_args(
     """
 
     def rename_startdate_enddate(df):
-        """ rename startdate, enddate to starttime endtime """
+        """rename startdate, enddate to starttime endtime"""
         col_set = set(df.columns)
         if "startdate" in col_set and "starttime" not in col_set:
             df = df.rename(columns={"startdate": "starttime"})
@@ -460,13 +462,13 @@ def get_waveforms_bulk_args(
         return df
 
     def _times_to_utc(df):
-        """ Convert time columns to UTCDateTime."""
+        """Convert time columns to UTCDateTime."""
         df["starttime"] = to_utc(df["starttime"])
         df["endtime"] = to_utc(df["endtime"])
         return df
 
     def _check_nslc_codes(df):
-        """ Ensure there are no wildcards in NSLC columns. """
+        """Ensure there are no wildcards in NSLC columns."""
         for code in NSLC:
             has_qmark = df[code].str.contains("?", regex=False).any()
             has_star = df[code].str.contains("*", regex=False).any()
@@ -476,7 +478,7 @@ def get_waveforms_bulk_args(
         return df
 
     def _check_starttime_endtime(df):
-        """ Ensure all starttimes are less than endtimes. """
+        """Ensure all starttimes are less than endtimes."""
         # starttimes must be <= endtime
         invalid_time_range = df["starttime"] >= df["endtime"]
         if invalid_time_range.any():
@@ -485,7 +487,7 @@ def get_waveforms_bulk_args(
         return df
 
     def _check_missing_data(df):
-        """ There should be no missing data in the required columns."""
+        """There should be no missing data in the required columns."""
         # first check if all required columns exist
         if not set(BULK_WAVEFORM_COLUMNS).issubset(set(df.columns)):
             missing_cols = set(BULK_WAVEFORM_COLUMNS) - set(df.columns)
